% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes. 
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

function [feat loc patches] = ImgFeature(img, feat_type, feat_loc, options)
%[feat loc patches] = ImgFeature(img, feat_type, feat_loc, options)
%   extract bag of feature from img
% feat_type: 'patch', 'sift', or 'phow'
% feat_loc: 'grid' or 'poi'
%
% feat: feature design matrix
% loc: location of the features. 
%   [x y width height] for patches
%   [x y scale orientation] for sift
% patches: the patches that generate the features

if nargin < 4; options = []; end
[color margin poi_nedge poi_ncorner grid_step patch_sizes ...
    sift_scales sift_orient phow_sizes phow_blank_thresh, ...
    show] = GetOptions(options, ...
    'color', false, 'margin', 16, ...
    'poi_nedge', 150, 'poi_ncorner', 50, 'grid_step', 20, ...
    'patch_sizes', 5:5:15, 'sift_scales', 1, 'sift_orient', 0,...
    'phow_sizes', 6:3:12, 'phow_blank_thresh',5e-3,...
    'show', false);

% preprocess
n_channel = size(img, 3);
if color && n_channel == 1
    warning('cannot extract color features. the image is grayscale');
end
if ~color && n_channel > 1
    img = rgb2gray(img);
end
img = single(img);
img = img - min(img(:));
img = img./max(img(:));

if strcmpi(feat_type, 'phow')
    [frames descrs] = vl_phow(img, 'Step', grid_step, 'Sizes', phow_sizes, ...
        'ContrastThreshold', phow_blank_thresh, ...
        'Color', iif(color, 'hsv', 'gray'));
    frames(3,:) = frames(4,:)/6; % set the correct sift scale
    frames(4,:) = 0; % set the orientation
    % loc: [x, y, scale, orientation]
    [feat loc] = deal(descrs', frames');
    
    if show
        idx = find(loc(:,3) == max(loc(:,3)));
        idx = idx(1:5:end);
        ShowSIFT(img, frames(:,idx), ...
            iif(n_channel == 1, descrs(:,idx), descrs(257:end,idx)));
    end
else
    if show; subplot(1,2,1); end
    switch lower(feat_loc)
        case 'poi'
            loc = GenInterestingPoints(img, poi_nedge, poi_ncorner, margin, show);
        case 'grid'
            loc = GenGridPoints(img, grid_step, margin, show);
        otherwise 
            error('unknown feature location');
    end

    if show; subplot(1,2,2); end
    switch lower(feat_type)
        case 'patch'
            feat = cell(1, length(patch_sizes));
            tblr = cell(1, length(patch_sizes));
            for ind = 1:length(patch_sizes)
                [feat{ind} tblr{ind}] = GetPatchFeature(img, loc, ...
                    struct('patch_radius', patch_sizes(ind), 'show', show));
            end
            feat = CatMat(feat, [], 1);
            tblr = CatMat(tblr, [], 1);
            % loc: [x, y, width, height]
            loc = round([mean(tblr(:,3:4),2) mean(tblr(:,1:2),2) tblr(:,4)-tblr(:,3) tblr(:,2)-tblr(:,1)]);
        case 'sift' % sift
            feat = cell(1, length(sift_scales)); 
            frames = cell(1, length(sift_scales));
            for ind = 1:length(sift_scales)
                [feat{ind} frames{ind}] = GetSIFTFeature(img, loc, ...
                    struct('scale',sift_scales(ind),'orient',sift_orient,'margin',margin));
            end
            feat = CatMat(feat, [], 1);
            % loc: [x, y, scale, orientation]
            loc = CatMat(frames, [], 1);
        otherwise
            error('unknown feature type');
    end

    filter = ~any(isnan(feat), 2);
    feat = feat(filter, :);
    loc = loc(filter, :);
end

if nargout > 2
    % get the image patch at each point
    patches = GetPatchFeature(img, round(loc), ...
        struct('feature_size',[21 21],'patch_radius',10));
end
