classdef PCA
    
    properties
        U % basis
        L % eigenvalues
        M % mean
        R % effective rank
        E % energy preserved
    end
    
    methods
        function [model] = PCA(X, E)
            if nargin > 0
                if nargin < 2
					E = 0.99;
                end
                model = PCA.Train(X, E);
            end
        end
        
        function [P] = Project(self, X, dims, whiten)
            if nargin < 4
                whiten = false;
                if nargin < 3
                    dims = 1:self.R;
                end
            end
            if isempty(dims)
                dims = 1:self.R;
            end
            P = bsxfun(@plus, X, -self.M)*self.U(:, dims);
            if whiten
                P = bsxfun(@times, P, 1./sqrt(self.L(dims)'));
            end
        end

        function [X] = Reconstruct(self, P)
            r = size(P, 2);
            X = bsxfun(@plus, P*self.U(:, 1:r)', self.M);
        end
    
        function [h] = Visualize(self, X, c, dim)
            if nargin < 4
                dim = 2;
                if nargin < 3
                    c = 'b';
                end
            end
            
            if dim == 2
                P = self.Project(X, 1:2);
                h = scatter(P(:,1), P(:,2), [], c, 'marker', '.');
            elseif dim == 3
                P = self.Project(X, 1:3);
                h = scatter3(P(:,1), P(:,2), P(:, 3), [], c, 'marker', '.');
            else
                error('cannot visualize in high dimensions');
            end
        end
    
        function [] = plot(self, pcs)
            if nargin < 2
                pcs = 1:2;
            end
            
            subplot(2,2,1);
            loglog(1:self.R, self.L(1:self.R), 'b', self.R:length(self.L), self.L(self.R:length(self.L)), 'r');
            title('Eigenvalues');
            subplot(2,2,2);plot(self.M);title('Data mean');
            subplot(2,2,3);plot(self.U(:,pcs(1)));title('First Basis');
            subplot(2,2,4);plot(self.U(:,pcs(2)));title('Second Basis');
        end
    end
    
    methods(Static)
        function model = Train(X, E)
            if nargin < 2
				E = 0.99;
            end
            
            model = PCA();
            X = double(X);
            
            [n, dim] = size(X);
            model.R = min(n, dim);
            model.M = mean(X, 1);
            X = bsxfun(@plus, X, -model.M);
            
            if n >= dim
                C=X'*X./n;
                [model.U S]=eig(C);
                [model.L I]=sort(diag(S), 'descend');
                model.U=model.U(:, I);
            else
                C=X*X'./n;
                [V S]=eig(C);
                [model.L I]=sort(diag(S), 'descend');
                model.U=X'*V(:, I)*diag(1./model.L./sqrt(n));
            end
            
            model.L = max(0, model.L);
            model.U = Normalize(model.U, 'n1', 1);
            model.E = E;
            model.R = EffectiveRank(model.L, model.E);
        end
    end
end
