% function [Pw_z,Pd_z,Pz,Li] = pLSA_EM(X,Fixed_Pw_z,K,Learn)
%
% Probabilistic Latent semantic alnalysis (pLSA)
%
% Notation:
% X ... (m x nd) term-document matrix (observed data)
%       X(i,j) stores number of occurrences of word i in document j
%
% m  ... number of words (vocabulary size)
% nd ... number of documents
% K  ... number of topics
%
% Fixed_Pw_z ... fixed Pw_z density (for recognition only)
%                leave empty in learning
%                N.B. Must be of size m by K
%
% Learn ... Structure holding all settings
%
% Li   ... likelihood for each iteration
% Pz   ... P(z)
% Pd_z ... P(d|z)
% Pw_z ... P(w|z) corresponds to beta parameter in LDA
%
% Pz_wd ... P(z|w,d) posterior on z
%
%
% References:
% [1] Thomas Hofmann: Probabilistic Latent Semantic Analysis,
% Proc. of the 15th Conf. on Uncertainty in Artificial Intelligence (UAI'99)
% [2] Thomas Hofmann: Unsupervised Learning by Probabilistic Latent Semantic
% Analysis, Machine Learning Journal, 42(1), 2001, pp.177.196
%
% Josef Sivic
% josef@robots.ox.ac.uk
% 30/7/2004
%
% Extended by Rob Fergus
% fergus@csail.mit.edu
% 03/10/05
%
% Optimized by Liang Xiong
% lxiong@cs.cmu.edu
% 11/18/11

function [Pw_z,Pd_z,Pz,Pz_d,Li] = pLSA_EM(X,K,options)

if nargin < 3; options = []; end
[Fixed_Pw_z, max_iter, epsilon, verbose] = GetOptions(options,...
    'Pw_z', [], 'max_iter', 1000, 'epsilon', 1e-2, 'verbose', true);

% small offset to avoid numerical problems
ZERO_OFFSET = 1e-7;

[m nd] = size(X);

% [Pz,Pd_z,Pw_z] = pLSA_init(m,nd,K);
[Pz,Pd_z,Pw_z] = pLSA_init_X(X,K);
Pz_dw = zeros(m,nd,K);

% if in recognition, reset Pw_z to fixed distribution
if ~isempty(Fixed_Pw_z)
    if all(size(Pw_z) == size(Fixed_Pw_z))
        Pw_z = Fixed_Pw_z;
    else
        error('provided topics are wrong.');
    end
end

Li = nan(max_iter, 1);
for it = 1:max_iter
    if verbose;
        fprintf('Iter %d ',it);
    end
    
    % E-step
    for k = 1:K
        Pz_dw(:,:,k) = Pw_z(:,k) * (Pd_z(:,k)' * Pz(k));
    end
    Pz_dw = bsxfun(@times, Pz_dw, 1./sum(Pz_dw,3));
    
    % M-step
    xpzdw = bsxfun(@times, X, Pz_dw);
    Pw_z = squeeze(sum(xpzdw,2));
    Pd_z = squeeze(sum(xpzdw,1));
    Pz = Normalize1(sum(Pd_z,1)')';
    Pw_z = Normalize1(Pw_z);
    Pd_z = Normalize1(Pd_z);
    
    % if in recognition mode, reset Pw_z to fixed density
    if ~isempty(Fixed_Pw_z)
        Pw_z = Fixed_Pw_z;
    end
    %%% avoid numerical problems.
    Pw_z = Normalize1(Pw_z + ZERO_OFFSET);
    
    % Evaluate data log-likelihood
    Li(it) = pLSA_logL(X,Pw_z,Pz,Pd_z);
    
    if verbose
        fprintf('L = %g\n', Li(it)/nd);
    end
    if it > 1 && (Li(it) - Li(it-1))/nd < epsilon
        break;
    end
end

Pdz = bsxfun(@times, Pd_z, Pz);
Pz_d = Normalize1(Pdz');

function [Pz,Pd_z,Pw_z] = pLSA_init(m,nd,K)
% random init

Pz   = ones(K,1)/K; % uniform prior on topics
Pd_z = Normalize1(rand(nd,K));   % word probabilities conditioned on topic
Pw_z = Normalize1(rand(m,K));

function [Pz,Pd_z,Pw_z] = pLSA_init_X(X,K)

[m nd] = size(X);

% seed the topics
n_try = 10;
idx = cell(1, n_try);
dist = zeros(1, n_try);
for ii = 1:n_try
    idx{ii} = ceil(rand(K, 1)*nd);
    dist(ii) = sum(sum(tril(L2Dist(X(:,idx{ii}), [], true, true))));
end
idx = idx{argmax(dist)};

% normalize
Pw_z = Normalize1(X(:, idx) + 1);
Pd_z = Normalize1(bsxfun(@times, X, 1./sum(X, 1))'*Pw_z + 1e-3);
Pz   = ones(K,1)/K; % uniform prior on topics

function L = pLSA_logL(X,Pw_z,Pz,Pd_z)

L = sum(sum(X.*log(bsxfun(@times,Pw_z,Pz) * Pd_z' + eps)));

function [A] = Normalize1(A)

s = 1./sum(A, 1);
s(isinf(s)) = 1;
A = bsxfun(@times, A, s);
