% ========================================================================
% Liang Xiong (lxiong[at]cs[dot]cmu[dot]edu), AutonLab, 2008-2013
%
% This piece of software is free for research purposes.
% We hope it was helpful but do not privide any warranty.
% If you encountered any problems please contact the author.
% ========================================================================

if matlabpool('size') == 0
    disp('WARNING: No matlabpool currently open. You might want to abort and open one, to make tuning go faster.');
end

addpath lib

% libsvm can be obtained from www.csie.ntu.edu.tw/~cjlin/libsvm
libsvm_path = '/auton/home/lxiong/h/matlab/Lib/libsvm-3.1/matlab';
addpath(libsvm_path);

% vlfeat can be obtained from www.vlfeat.org
vlfeat_path = '/auton/home/lxiong/h/matlab/Lib/vlfeat-0.9.14/toolbox';
cwd = pwd; cd(vlfeat_path); vl_setup; cd(cwd)

%% the following settings can be customized
% data_dir: folder that holds all the data. images from each class should
%   be in a separate folder
% class_names: names of the classes/folders
%
% data_name: name of the data set
% img_per_cla: number of images per class to use
% color: use color feature or not
% grid_step: grid step size of the feature sampler
% phow_sizes: scales of the phow descriptors
% pca_dim: variance to keep
% spatial_info: spatial info to add. '' for none, 'y' for y coordinates.

data_dir = './images';
MIT = {'MITcoast','MITforest', 'MIThighway', 'MITinsidecity',...
    'MITmountain','MITopencountry','MITstreet','MITtallbuilding'};
class_names = MIT;

opt = struct('data_name', 'mit', 'n_class', length(class_names), 'show', false);
opt.data_opt = struct(...
    'img_per_cla', 50, 'sampler', 'first',...
    'color', false, 'feat_type', 'phow', 'feat_loc', 'grid',...
    'grid_step', 10, 'phow_sizes', 6:3:12, 'phow_blank_thresh', 0);
opt.feat_opt = struct(...
    'blank_handler', 'fill',...
    'pca', true, 'pca_dim', 0.7,...
    'spatial_info', '',...
    'normalizer', 'm0v1'); % normalize to mean 0, variance 1
opt.bow_opt = struct('n_code', 500, 'n_topic', 25);
opt.div_opt = struct('k', 5); % computation-related settings around line 124...
opt.div_opt.div_funcs = {'renyi:.9', 'renyi:.99', 'hellinger', 'l2', 'bc'};
opt.div_opt.div_names = {'NP_R_9', 'NP_KL', 'NP_H', 'NP_L2', 'BC'};
opt.cla_opt = struct('n_try', 8, 'n_fold', 2);

% uncomment if you want the full data set
% opt.data_opt.img_per_cla = 500;

% uncomment if you want color sift
% opt.data_opt.color = true;

% uncomment if you want the patches' vertical locations as a feature
% opt.feat_opt.spatial_info = 'y';

%% extract feature

if ~exist(ImgFileData(opt), 'file')
    disp('Loading images...');
    [imgs, labels, files] = LoadImageData(data_dir, class_names, ...
        opt.data_opt.img_per_cla, opt.data_opt.sampler);

    disp('Extracting features...');
    [features_raw locs patches] = ImgExtractFeature(imgs, opt);

    fprintf('Saving data to %s\n', ImgFileData(opt));
    save(ImgFileData(opt));
else
    opt__ = opt;
    load(ImgFileData(opt));
    opt = opt__;
    clear opt__;
end

%% compute the bow representation

if ~exist(ImgFileBoW(opt), 'file')
    disp('Discretizing data...');
    [X_bow X_plsa codebook codes] = ImgMakeBoW(features_raw, imgs, locs, opt);

    fprintf('Saving bow data to %s\n', ImgFileBoW(opt));
    save(ImgFileBoW(opt), 'opt', 'codebook', 'codes', 'X_bow', 'X_plsa');
else
    opt__ = opt;
    load(ImgFileBoW(opt));
    opt = opt__;
    clear opt__;
end

%% make bof representation

if ~exist(ImgFileFeat(opt), 'file')
    features_raw = map(@(x) single(x), features_raw);
    locs = map(@(x) single(x), locs);
    [features pca] = ImgMakeBoF(features_raw, imgs, locs, opt);

    clear features_raw imgs locs
    fprintf('Saving bof data to %s\n', ImgFileFeat(opt));
    save(ImgFileFeat(opt));
else
    opt__ = opt;
    load(ImgFileFeat(opt));
    opt = opt__;
    clear opt__;
end
% now 'features' contains the bag-of-features representations

clear features_raw imgs
M = length(features);

%% compute divergences

if ~exist(ImgFileDiv(opt), 'file')
    dop = opt.div_opt;
    dop.index = 'linear';
    dop.num_threads = 12;

    disp('Computing NP divergences...');
    npDs = NPDivs(features, [], dop.div_funcs, dop);

    save(ImgFileDiv(opt), 'dop', 'opt', 'npDs');
else
    opt__ = opt;
    load(ImgFileDiv(opt));
    opt = opt__;
    clear opt__;
end

Ds = cell(1, 2); div_names = cell(1, 2);
Ds{1} = ChiDist(X_bow); div_names{1} = 'BoW';
Ds{2} = L2Dist(X_plsa); div_names{2} = 'PLSA';
Ds = [Ds npDs]; div_names = [div_names opt.div_opt.div_names];

Ds = cat(3, Ds{:});

%% run classification

disp('Starting classification...');
result_file = ImgFileCla(opt);

n_try = opt.cla_opt.n_try;
results = cell(1, n_try);
parfor tnd = 1:n_try
    disp(tnd);
    SeedRand(tnd);

    cvo = cvpartition(M, 'kfold', opt.cla_opt.n_fold);
    results{tnd} = ExecCla(labels, Ds, div_names, cvo);
end
save(result_file, '-v7.3');

%% show results

score_tags = map(@(n) sprintf('svm_%s',n), div_names);
score_names = div_names;
n_score = length(score_tags);

accs = nan(n_score, length(results));
for ind = 1:n_score
    accs(ind, :) = CatVec(results, ['acc_' score_tags{ind}]);
end

figure;
boxplot(accs');
SetAxis(gca,[],'Accuracy',[],[],1:n_score,score_names);

mean_accs = mean(accs');
std_accs = std(accs');
for ind = 1:length(score_names)
    fprintf('%15s : %0.4f +- %0.4f\n', div_names{ind}, mean_accs(ind), 3*std_accs(ind));
end
